"use strict";
/*
 * MIT License
 *
 * Copyright (c) 2018 Choko (choko@curioswitch.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
}
Object.defineProperty(exports, "__esModule", { value: true });
const immutable_1 = require("immutable");
const react_router_redux_1 = require("react-router-redux");
const redux_1 = require("redux");
const redux_saga_1 = __importDefault(require("redux-saga"));
const reducer_1 = __importDefault(require("../containers/LanguageProvider/reducer"));
const reducers_1 = __importDefault(require("./reducers"));
const sagaMiddleware = redux_saga_1.default();
function identityReducer(state) {
    return state;
}
const nonInjectedReducers = {
    language: reducer_1.default,
    route: reducers_1.routeReducer,
};
function configureStore(initialState, history) {
    // Create the store with two middlewares
    // 1. sagaMiddleware: Makes redux-sagas work
    // 2. routerMiddleware: Syncs the location/URL path to the state
    const middlewares = [sagaMiddleware, react_router_redux_1.routerMiddleware(history)];
    const enhancers = [redux_1.applyMiddleware(...middlewares)];
    const windowCast = window ? window : null;
    // If Redux DevTools Extension is installed use it, otherwise use Redux compose
    const composeEnhancers = process.env.NODE_ENV !== 'production' &&
        windowCast &&
        windowCast.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__
        ? windowCast.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__({
            // TODO Try to remove when `react-router-redux` is out of beta, LOCATION_CHANGE should not be fired more than once after hot reloading
            // Prevent recomputing reducers for `replaceReducer`
            shouldHotReload: false,
        })
        : redux_1.compose;
    // NOTE(choko): Cannot use spread with a generic type yet - https://github.com/Microsoft/TypeScript/pull/13288
    // tslint:disable-next-line:prefer-object-spread
    const merged = Object.assign({}, {
        language: reducer_1.initialState,
        route: reducers_1.routeInitialState,
    }, initialState);
    const globalInitialState = immutable_1.Record(merged)();
    const identityReducers = globalInitialState
        .toSeq()
        .keySeq()
        .filter((key) => !(key in Object.keys(nonInjectedReducers)))
        .reduce((reducers, key) => ({
        ...reducers,
        [key]: identityReducer,
    }), {});
    const store = redux_1.createStore(reducers_1.createInitalReducer(identityReducers, nonInjectedReducers), 
    // TODO(choko): Figure out what's wrong with the types that causes the need for this force-cast.
    globalInitialState, composeEnhancers(...enhancers));
    // Extensions
    store.runSaga = sagaMiddleware.run;
    store.identityReducers = identityReducers;
    store.nonInjectedReducers = nonInjectedReducers;
    store.injectedReducers = {}; // Reducer registry
    store.injectedSagas = {}; // Saga registry
    // Make reducers hot reloadable, see http://mxs.is/googmo
    /* istanbul ignore next */
    if (module.hot) {
        module.hot.accept('./reducers', () => {
            store.replaceReducer(reducers_1.default(store));
        });
    }
    return store;
}
exports.default = configureStore;
//# sourceMappingURL=store.js.map